//--------------------------------------------------------------------------------------
//  The Header Guard thingie plugin for Eddie
//  (C) Alan Ellis 1998 (alan@cgsoftware.com)
//
//  Unauthorized duplication is restricted, therefore I am authorizing you to duplicate
// this. =)
//
//
//--------------------------------------------------------------------------------------


#include <Alert.h>
#include <Bitmap.h>
#include <ctype.h>
#include <SmartArray.h>
#include <PopUpMenu.h>
#include <MenuItem.h>

#include "Plugin.h"
#include "PiperBitmaps.h"

class PiperPrimitive
{
public:
	PiperPrimitive() : cmd(""), init(false)
	{ }




	PiperPrimitive(char const *namestr, char const *cmdstr)
			: name(namestr), cmd(cmdstr), init(true)
	{ }




	PiperPrimitive(BString const &namebstr, BString const &cmdbstr)
			: name(namebstr), cmd(cmdbstr), init(true)
	{ }



	BString name;
	BString cmd;
	bool init;
};

void PipeSelection(PluginInterface *pluginInterface, PiperPrimitive usePiper);


int kNumPipers = 10;

SmartArray<PiperPrimitive> gPiperPrimitives(kNumPipers);

void PiperMenu(PluginInterface *pi)
{
	BPopUpMenu *completionMenu = new BPopUpMenu("pipers menu",false,false);

	BMenuItem *item=NULL;
	for(int i=0; i<gPiperPrimitives.Count(); i++) {
		if (gPiperPrimitives[i].init)
			completionMenu->AddItem(item=new BMenuItem(gPiperPrimitives[i].name.String(),NULL));
	}

	// Get the result
	BPoint point=pi->GetButtonRect().LeftBottom();
	pi->Window()->ConvertToScreen(&point);
	item=completionMenu->Go(point);

	if (item!=NULL) {
		//		if (modifiers()&B_SHIFT_KEY) {
		//			c.erase(item->Label());
		//			if (modifiers()&B_COMMAND_KEY) d.erase(item->Label());
		//		} else pi->Insert(item->Label(), strlen(item->Label()));
		PipeSelection(pi, gPiperPrimitives[completionMenu->IndexOf(item)]);
	}
}



// PiperPrimitive gPiperPrimitives[kNumPipers];

PluginInterface::PluginResult
PluginMain(PluginInterface::PluginCallSelector selector,
           PluginInterface *pluginInterface)
{
	PluginInterface::PluginResult result = PluginInterface::kIgnored;

#if 0
	fprintf(stderr, "Piper: PluginInterface::PluginResult PluginMain() called\n");
#endif

	char const **argv;

	int idx;
	char const *cmd;
	char const *name;
	BString newname;

	switch (selector) {

		case PluginInterface::kGetPluginVersion:

			pluginInterface->ReturnSupportedPluginVersion();
			pluginInterface->ReturnRequiredPluginVersion();
			result = PluginInterface::kAccepted;
			break;

		case PluginInterface::kInit:

			// publish the editor primitives used by the shortcuts:





			pluginInterface->RegisterHandledShellCommand("SetPiper");

			pluginInterface->RegisterHandledPrimitiveCommand("Piper0", "User-defined text filter 0");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper1", "User-defined text filter 1");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper2", "User-defined text filter 2");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper3", "User-defined text filter 3");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper4", "User-defined text filter 4");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper5", "User-defined text filter 5");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper6", "User-defined text filter 6");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper7", "User-defined text filter 7");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper8", "User-defined text filter 8");
			gPiperPrimitives.AddItem(PiperPrimitive());
			pluginInterface->RegisterHandledPrimitiveCommand("Piper9", "User-defined text filter 9");
			gPiperPrimitives.AddItem(PiperPrimitive());

			// pluginInterface->RegisterHandledPrimitiveCommand("PipeSelection",
			//        "Filter the selection through an external program");

			result = PluginInterface::kAccepted;

#ifdef DEBUG
			fprintf(stderr, "Piper: case PluginInterface::kInit:\n");
#endif

			break;

		case PluginInterface::kArgvPassed:
#ifdef DEBUG
			fprintf(stderr, "Piper: case PluginInterface::kArgvPassed:\n");
#endif
			argv = *(pluginInterface->CurrentArgv());
			if (strcmp(argv[0], "SetPiper") == 0) {
				idx = atoi(argv[1]);
				name = argv[2];
				cmd = argv[3];
				if (idx < 0) {
					fprintf(stderr, "SetPiper: error: '%s' not in range\n", argv[1]);
				} else {
					gPiperPrimitives.RemoveItem(idx);
					gPiperPrimitives.InsertItem(PiperPrimitive(name, cmd), idx);
#ifdef DEBUG
					fprintf(stderr, "Piper: installing Piper%d => '%s'\n", idx, cmd);
#endif
				}
				result = PluginInterface::kAccepted;
			}

			break;

		case PluginInterface::kWindowOpened:

			pluginInterface->SetupPluginButton(kButtonBits, kPressedButtonBits);

			result = PluginInterface::kAccepted;
			break;

		case PluginInterface::kButtonPress:

			// PipeSelection(pluginInterface);



			PiperMenu(pluginInterface);

			result = PluginInterface::kAccepted;
			break;

		case PluginInterface::kButtonClick:

			PipeSelection(pluginInterface, gPiperPrimitives[0]);

			result = PluginInterface::kAccepted;
			break;


		case PluginInterface::kPrimitiveInvoked:
			{
				// handle a primitive from a keyboard command
				const char *primitive = pluginInterface->CurrentPrimitive();

				// make sure our shortcut was called
				//				if (strcmp(primitive, "PipeSelection") == 0) {
				//					// animate the button as if it was pressed by a mouse to give the user
				//					// a positive feedback
				//					pluginInterface->AnimateButtonClick();
				//
				//					PipeSelection(pluginInterface);
				//				}
				if (!(strcmp(primitive, "Piper0")
				        &&strcmp(primitive, "Piper1")
				        &&strcmp(primitive, "Piper2")
				        &&strcmp(primitive, "Piper3")
				        &&strcmp(primitive, "Piper4")
				        &&strcmp(primitive, "Piper5")
				        &&strcmp(primitive, "Piper6")
				        &&strcmp(primitive, "Piper7")
				        &&strcmp(primitive, "Piper8")
				        &&strcmp(primitive, "Piper9"))) {
					PiperPrimitive piper;
					idx = (primitive[strlen("Piper")] - '0');
					if (gPiperPrimitives[idx].init) {
						// animate the button as if it was pressed by a mouse to give the user
						// a positive feedback
						pluginInterface->AnimateButtonClick();

						PipeSelection(pluginInterface, gPiperPrimitives[idx]);
					}
					result = PluginInterface::kAccepted;
				}
				break;
			}
			// appease the compiler warning gods and fall through a bunch of cases
		case PluginInterface::kUnload      :
		case PluginInterface::kQuit        :
		case PluginInterface::kButtonDraw  :
		case PluginInterface::kPulse       :
		case PluginInterface::kDraw        :
		case PluginInterface::kMessage     :
		case PluginInterface::kMouseDown   :
		case PluginInterface::kKeyDown     :
		case PluginInterface::kWindowActivated      :
		case PluginInterface::kWindowDeactivated    :
		case PluginInterface::kDocumentTextChanged  :
		case PluginInterface::kDocumentOpened       :
		case PluginInterface::kDocumentClosed       :
		case PluginInterface::kDocumentStateChanged :
		case PluginInterface::kDocumentActivated    :
		case PluginInterface::kWindowClosed         :
		case PluginInterface::kGlobalSettingsSaved  :
		case PluginInterface::kDocumentSettingsSaved:
		case PluginInterface::kGlobalPrefsShowing   :
		case PluginInterface::kSyntaxColoringOff    :
		case PluginInterface::kFindQuotedString     :
			result = PluginInterface::kIgnored;
			break;
			// something is wrong/new/unhandled
		default:
			fprintf(stderr, "Piper: Wrong/New/Unhandled case, selector value: %d\n", selector);
			result = PluginInterface::kIgnored;
			break;
	}
	return result;
}


const char *
PluginHelp(void)
{
	return "Piper";
}
//---------------------------------------------------------------------
// dsandler's stuff
//---------------------------------------------------------------------
void PipeSelection(PluginInterface *pluginInterface, PiperPrimitive usePiper)
{
	// Is this a header file?
	// If not bail
	//	if (pluginInterface->LanguageType() != kCH)
	//		if (pluginInterface->LanguageType() != kCHPlus)
	//		{
	//			fprintf(stderr, "Piper: Not a C/C++ header file according to Eddie.\n");
	//			return;
	//		}

	int32 SelStart,
	SelEnd;

	pluginInterface->GetSelection(&SelStart, &SelEnd);

	BString tmpfile("/tmp/Eddie.Piper.tmp.");
	tmpfile << (long)(find_thread(NULL)); // get the thread ID for a good filename

	char *buffer = new char[(SelEnd-SelStart)+1];

	if (!buffer) {
		(new BAlert("Piper alert", "Out of memory.", "OK"))->Go();
		return;
	}

	pluginInterface->GetText(buffer, SelStart, (SelEnd-SelStart));

	//	BString cmd("/boot/home/config/bin/astyle -ctlS > '");
	//	BString cmd("/boot/home/config/bin/tidy -config '/boot/apps/HTML TIDY/config.txt' > '");
	BString cmd(usePiper.cmd);
	cmd << " > '" << tmpfile << "'";

#if DEBUG
	fprintf(stderr, "Piper: using command: %s\n", cmd.String());
#endif

	// Run the filter; save the output.

	FILE *cmdpipe = popen(cmd.String(), "w");

	ssize_t written = fwrite((void *)buffer, 1, (SelEnd-SelStart), cmdpipe);

	delete [] buffer;
	buffer = NULL;

	if (written != (SelEnd-SelStart)) {
		unlink(tmpfile.String());
		(new BAlert("Piper alert", "There was a problem saving the filter output.", "OK"))->Go();
		return;
	}

	status_t resultCode = pclose(cmdpipe);

	if (resultCode != B_OK) {
		BString errorMsg("The filter returned an error: ");
		errorMsg << strerror(resultCode);
		(new BAlert("Piper alert", errorMsg.String(), "OK"))->Go();
		unlink(tmpfile.String());
		return;
	}

	// Read the filter output back in.

	int fd = open(tmpfile.String(), O_RDONLY);

	if (fd < 0) {
		unlink(tmpfile.String());
		(new BAlert("Piper alert", "Couldn't read temp file.", "OK"))->Go();
		return;
	}

	struct stat fileinfo;
	fstat(fd, &fileinfo);
	ssize_t datasize = fileinfo.st_size;

	if (datasize <= 0) {
		close(fd);
		unlink(tmpfile.String());
		(new BAlert("Piper alert", "Temp file had no data.", "OK"))->Go();
		return;
	}

	buffer = new char[datasize+1];

	if (!buffer) {
		close(fd);
		unlink(tmpfile.String());
		(new BAlert("Piper alert", "Out of memory.", "OK"))->Go();
		return;
	}

	ssize_t datalength = read(fd, (void *)buffer, datasize);
	buffer[datalength] = '\0';

	close(fd);

	// Make the changes to the document.

	pluginInterface->StartUndo(SelStart, SelEnd, "Piper");

	pluginInterface->CurrentUndoInsert(buffer, datalength); // at selection start

	//    pluginInterface->CurrentUndoRemove(SelStart, (SelEnd-SelStart));
	//    pluginInterface->CurrentUndoInsert(buffer, datalength, SelStart);
	//    pluginInterface->Select(SelStart, datalength);

	delete [] buffer;

	pluginInterface->SealUndo(SelStart, SelStart+datalength);

	unlink(tmpfile.String());
}

//--------------------------------------------------------------------
